-- Creacin de tablas 

CREATE TABLE Categora
(
  id_categora    NUMBER(8)         NOT NULL,
  nombre          VARCHAR2(20)      NOT NULL,
  descripcin     VARCHAR2(100),
  
  CONSTRAINT PK_Categora PRIMARY KEY (id_categora),
  CONSTRAINT UK_Categora UNIQUE (nombre)
);

CREATE TABLE Proveedor
(
  id_proveedor     NUMBER(8)         NOT NULL,
  nombre           VARCHAR2(20)      NOT NULL,
  
  CONSTRAINT PK_proveedor PRIMARY KEY (id_proveedor),
  CONSTRAINT UK_proveedor UNIQUE (nombre)
);

CREATE TABLE Artculo
(
  id_artculo         NUMBER(8)         NOT NULL,
  categora           NUMBER(8)         NOT NULL,
  descripcin         VARCHAR2(40)      NOT NULL,
  precio              NUMBER(12,2)      NOT NULL,
  IVA                 NUMBER(2)         NOT NULL,
  proveedor           NUMBER(8)         NOT NULL,
  
CONSTRAINT PK_Artculo PRIMARY KEY (id_artculo),
CONSTRAINT FK_Artculo_Categora FOREIGN KEY (categora) REFERENCES Categora (id_categora),
CONSTRAINT CK_articulo CHECK (IVA IN( '4', '10','21')),
CONSTRAINT FK_Artculo_Proveedor FOREIGN KEY (proveedor) REFERENCES Proveedor (id_proveedor)
);

CREATE TABLE Miembro
(
  id_miembro  NUMBER(8)         NOT NULL,
  email       VARCHAR2(30)      NOT NULL,
  nombre      VARCHAR2(20)      NOT NULL,
  apellido1   VARCHAR2(20)      NOT NULL,
  apellido2   VARCHAR2(20)      NOT NULL,
  direccin   VARCHAR2(40)      NOT NULL,
  ciudad      VARCHAR2(40)      NOT NULL,
  pas        VARCHAR2(40)      NOT NULL,
  
  CONSTRAINT PK_Miembro PRIMARY KEY (id_miembro),
  CONSTRAINT UK_Miembro UNIQUE (email)
);


CREATE TABLE Desea
(
  id_deseo     NUMBER(8)         NOT NULL,
  homenajeado  NUMBER(8)         NOT NULL,
  artculo     NUMBER(8)         NOT NULL,
  fecha_alta   DATE              NOT NULL,
  
  CONSTRAINT PK_Desea PRIMARY KEY (id_deseo),
  CONSTRAINT FK_Desea_Miembro FOREIGN KEY (homenajeado) REFERENCES Miembro(id_miembro),
  CONSTRAINT FK_Desea_Artculo FOREIGN KEY (artculo) REFERENCES Artculo(id_artculo),
  CONSTRAINT UK_Desea UNIQUE (homenajeado, artculo)
);

CREATE TABLE Grupo
(
  id_grupo                  NUMBER(8)            NOT NULL,
  homenajeado               NUMBER(8)            DEFAULT'0',
  receptor                  NUMBER(8)            DEFAULT'0',
  dedicatoria_grupal        VARCHAR2(60)         NOT NULL,
  
  CONSTRAINT PK_Grupo PRIMARY KEY (id_grupo)
);

CREATE TABLE Agrupa
(
  email                      VARCHAR2(30)      NOT NULL,
  grupo                      NUMBER(8)         NOT NULL,
  dedicatoria_personal       VARCHAR2(40),
  
  CONSTRAINT PK_Agrupa PRIMARY KEY (email, grupo),  
  CONSTRAINT FK_Agrupa_Miembro FOREIGN KEY (email) REFERENCES Miembro(email),
  CONSTRAINT FK_Agrupa_Grupo FOREIGN KEY (grupo) REFERENCES Grupo(id_grupo)
  
);

CREATE TABLE Regala
(
  id_regalo                   NUMBER            NOT NULL, 
  grupo                       NUMBER            NOT NULL,
  deseo                       NUMBER            NOT NULL,
  fecha_inicio_recaudacin    DATE              NOT NULL,
  fecha_final_recaudacin     DATE              NOT NULL,
  estado_regalo               VARCHAR2(30)      DEFAULT 'AFEGIT_A_UN_GRUP'     NOT NULL,

  CONSTRAINT PK_Regala PRIMARY KEY (id_regalo),
  CONSTRAINT FK_Regala_Grupo FOREIGN KEY (grupo) REFERENCES Grupo(id_grupo),
  CONSTRAINT FK_Regala_Desea FOREIGN KEY (deseo) REFERENCES Desea(id_deseo),
  CONSTRAINT FK_Regala UNIQUE (grupo,deseo),
  CONSTRAINT CK_Regala CHECK (fecha_final_recaudacin >= fecha_inicio_recaudacin),
  CONSTRAINT CK_Regala1 CHECK (estado_regalo IN('AFEGIT_A_UN_GRUP', 'ELIMINAT','NO_SUFICIENTS_FONS',
                                        'PAGAMENT_COMPLETAT','ENVIAT','REBUT'))
);

CREATE TABLE Mtodo
(
  id_mtodo      NUMBER(8)         NOT NULL,
  mtodo         VARCHAR2(50)      NOT NULL,
  
  CONSTRAINT PK_Mtodo PRIMARY KEY (id_mtodo)
);

CREATE TABLE Transaccin
(
  id_transaccin    NUMBER            NOT NULL,
  regalo            NUMBER            NOT NULL,
  aportador         NUMBER            NOT NULL,
  cantidad          NUMBER            NOT NULL,
  tipo              VARCHAR2(30)      NOT NULL,
  traza_abono       NUMBER            NULL,
  fecha             DATE              NOT NULL,
  forma_pago        NUMBER            NOT NULL,
  visible           VARCHAR2(30)      NOT NULL,
  
  CONSTRAINT PK_Transaccin PRIMARY KEY (id_transaccin),
  CONSTRAINT FK_Transaccin_Regala FOREIGN KEY (regalo)  REFERENCES Regala(id_regalo),
  CONSTRAINT FK_Transaccin_Miembro FOREIGN KEY (aportador) REFERENCES Miembro (id_miembro),
  CONSTRAINT FK_Transaccin_Mtodo FOREIGN KEY (forma_pago) REFERENCES Mtodo(id_mtodo),
  CONSTRAINT CK_Transaccin CHECK (tipo IN('PAGO', 'ABONO')),
  CONSTRAINT CK_Transaccin1 CHECK (visible IN('SI','NO'))
);

CREATE TABLE EST1_Veces_Regalado
(
  producto        NUMBER(8)           NOT NULL,
  ao             NUMBER(4)           NOT NULL,
  numero_veces    NUMBER(4)           NOT NULL,
  
  CONSTRAINT PK_EST1_Veces_Regalado PRIMARY KEY (producto,ao)
);

CREATE TABLE EST2_Total_Regalos
(
  ao             NUMBER(4)           NOT NULL,
  total_regalos   NUMBER(8)           NOT NULL,
  
  CONSTRAINT PK_EST2_Veces_Regalado PRIMARY KEY (ao)
);

CREATE TABLE EST3_Importe_Total
(
  ao             NUMBER(4)             NOT NULL,
  importe_total   NUMBER(8,2)           NOT NULL,
  
  CONSTRAINT PK_EST3_Importe_Total PRIMARY KEY (ao)
);

CREATE TABLE EST4_Porcentaje_Completado
(
  ao              NUMBER(4)            NOT NULL,
  porcentaje       NUMBER(12,4)          NOT NULL,
  
  CONSTRAINT PK_EST4_Porcentaje_Completado PRIMARY KEY (ao)
);

CREATE TABLE EST5_Media_gasto
(
  ao             NUMBER(4)           NOT NULL,
  media_gasto     NUMBER(4,2)         NOT NULL,
  
  CONSTRAINT PK_EST5_Media_Gasto PRIMARY KEY (ao)
);

CREATE TABLE EST6_Mas_Homenajeado
(
  ao             NUMBER(4)           NOT NULL,
  mes             NUMBER(2)           NOT NULL,
  id_miembro      NUMBER              NOT NULL,
  n_veces         NUMBER              NOT NULL
  
  --CONSTRAINT EST6_Mas_Homenajeado PRIMARY KEY (ao,mes)
);

CREATE TABLE EST7_Mas_Gastador
(
  ao            NUMBER(4)            NULL,
  mes            NUMBER(2)            NULL,
  id_miembro     NUMBER               NULL,
  cantidad       NUMBER               NULL
  
  
  ---CONSTRAINT EST7_Mas_Gastador PRIMARY KEY (ao,mes)
);

CREATE TABLE EST8_Mas_Participativo
(
  ao           NUMBER(4)             NOT NULL,
  mes           NUMBER(2)             NOT NULL,
  id_miembro    NUMBER                NULL,
  n_veces       NUMBER                NULL
  
  ---CONSTRAINT EST8_Mas_Participativo PRIMARY KEY (ao,mes,id_miembro)
);

CREATE TABLE EST9_Grupo_Mas_Gastador
(
  ao           NUMBER(4)             NOT NULL,
  mes           NUMBER(2)             NOT NULL,
  id_grupo      NUMBER                NULL,
  total         NUMBER                NULL
  
  --CONSTRAINT EST9_Grupo_Mas_Gastador PRIMARY KEY (ao,mes,id_grupo)
);

CREATE TABLE Logs
(
   Id_Log               INTEGER NOT NULL,
   Procedimiento        VARCHAR2(200) NOT NULL,
   Parametros_Entrada   VARCHAR2(500) NOT NULL,
   Parametros_Salida    VARCHAR2(500) NOT NULL,
   Fecha                DATE NOT NULL,
   
   CONSTRAINT PK_Log PRIMARY KEY (Id_Log)
);
