CREATE OR REPLACE PROCEDURE TFC.SP_EST1_VECES_REGALADO (
   p_producto              IN EST1_VECES_REGALADO.producto%TYPE,
   p_ao                   IN EST1_VECES_REGALADO.ao%TYPE,   
   RSP                     OUT VARCHAR2)
   
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST1_VECES_REGALADO
   Obejtivo:               actualiza la tabla EST1_VECES_REGALADO
                   
   Parmetros de entrada:  p_producto, p_ao            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_numero_veces       INTEGER;
v_integridad         INTEGER;
v_existe_registro    INTEGER;
E_CONOCIDA           EXCEPTION; 

BEGIN
   v_parametros_entrada:='p_producto: '|| to_char(p_producto)
                      || 'p_ao: '     || to_char(p_ao);
   
  -- Comprueba si los parmetros de entrada son nulos
   IF (p_producto IS NULL) THEN 
     RSP:='El campo p_producto no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
   
   IF (p_ao IS NULL) THEN 
     RSP:='El campo p_ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si el producto existe
   SELECT COUNT(*) INTO v_integridad FROM Artculo WHERE id_artculo=p_producto;
   
   IF (v_integridad=0) THEN
     RSP:='No existe ninguna producto con ese cdigo';
     RAISE E_CONOCIDA;
   END IF;
   
   --Comprobamos si existe el registro en la tabla
  SELECT COUNT(*) INTO v_existe_registro FROM Est1_veces_regalado WHERE producto=p_producto AND ao=p_ao;
  
  IF v_existe_registro=0 THEN
     v_numero_veces:=1;
     INSERT INTO Est1_veces_regalado VALUES (p_producto,p_ao,v_numero_veces);
  END IF;
  
  IF v_existe_registro!=0 THEN
    SELECT numero_veces INTO v_numero_veces FROM Est1_veces_regalado WHERE producto=p_producto AND ao=p_ao;
     v_numero_veces:=v_numero_veces+1;
     UPDATE EST1_Veces_Regalado SET numero_veces=v_numero_veces WHERE producto=p_producto AND ao = p_ao; 
  END IF;
  
  -- terminado ok
   RSP:= 'OK_EST1_VECES_REGALADO';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST1_VECES_REGALADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST1_VECES_REGALADO: ' || RSP;
        SP_ALTA_LOG('SP_EST1_VECES_REGALADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR SP_EST1_VECES_REGALADO: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST1_VECES_REGALADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST1_VECES_REGALADO;
/
