CREATE OR REPLACE PROCEDURE TFC.SP_EST2_TOTAL_REGALOS (
   p_ao                   IN EST2_TOTAL_REGALOS.ao%TYPE,   
   RSP                     OUT VARCHAR2)
   
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST2_TOTAL_REGALOS
   Obejtivo:               actualiza la tabla EST2_TOTAL_REGALOS
                   
   Parmetros de entrada:  p_ao            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_numero_veces       INTEGER;
v_existe_registro    INTEGER;
E_CONOCIDA           EXCEPTION; 

BEGIN
   v_parametros_entrada:='p_ao: '     || to_char(p_ao);
   
   --Comprobamos que el parmtro de entrada no sea nulo
   IF (p_ao IS NULL) THEN 
     RSP:='El campo p_ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
   
   --Comprobamos si existe el registro en la tabla
  SELECT count(*) INTO v_existe_registro FROM Est2_total_regalos WHERE ao=p_ao;
  
  --Introducimos los datos en la tabla
  IF v_existe_registro=0 THEN
     v_numero_veces:=1;
     INSERT INTO Est2_total_regalos VALUES (p_ao,v_numero_veces);
  END IF;
  
  IF v_existe_registro!=0 THEN
    SELECT total_regalos INTO v_numero_veces FROM Est2_total_regalos WHERE ao=p_ao;
     v_numero_veces:=v_numero_veces+1;
     UPDATE Est2_total_regalos SET total_regalos=v_numero_veces WHERE ao = p_ao; 
  END IF;
  
  -- terminado ok
   RSP:= 'OK_EST2_TOTAL_REGALOS';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST2_VECES_REGALADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST2_VECES_REGALADO: ' || RSP;
        SP_ALTA_LOG('SP_EST2_VECES_REGALADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR SP_EST2_VECES_REGALADO: ' || v_err_msg;
        SP_ALTA_LOG('SP_SP_EST2_VECES_REGALADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST2_TOTAL_REGALOS;
/
