CREATE OR REPLACE PROCEDURE TFC.SP_EST3_IMPORTE_TOTAL (
   p_producto              IN Artculo.id_artculo%TYPE,
   p_ao                   IN EST3_IMPORTE_TOTAL.ao%TYPE,   
   RSP                     OUT VARCHAR2)
   
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST3_IMPORTE_TOTAL
   Obejtivo:               actualiza la tabla EST3_IMPORTE_TOTAL
                   
   Parmetros de entrada:  p_producto, p_ao            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_importe_total      NUMBER;
v_integridad         INTEGER;
v_existe_registro    INTEGER;
v_precio             NUMBER;
E_CONOCIDA           EXCEPTION; 

BEGIN
   v_parametros_entrada:='p_producto: '|| to_char(p_producto)
                      || 'p_ao: '     || to_char(p_ao);
   
  -- Comprueba si los parmetros de entrada son nulos
   IF (p_producto IS NULL) THEN 
     RSP:='El campo p_producto no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
   
   IF (p_ao IS NULL) THEN 
     RSP:='El campo p_ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si el producto existe
   SELECT COUNT(*) INTO v_integridad FROM Artculo WHERE id_artculo=p_producto;
   
   IF (v_integridad=0) THEN
     RSP:='No existe ninguna producto con ese cdigo';
     RAISE E_CONOCIDA;
   END IF;
   
  --Obtenemos el precio del producto 
  SELECT precio INTO v_precio FROM Artculo WHERE id_artculo=p_producto;

  --Comprobamos si existe el registro en la tabla
  SELECT COUNT(*) INTO v_existe_registro FROM Est3_importe_total WHERE ao=p_ao;
   
  IF v_existe_registro=0 THEN
     v_importe_total:=v_precio;
     INSERT INTO Est3_importe_total VALUES (p_ao, v_importe_total);
  END IF;
  
  IF v_existe_registro!=0 THEN
    SELECT importe_total INTO v_importe_total FROM Est3_importe_total WHERE ao=p_ao;
    v_importe_total:=v_importe_total+v_precio;

  UPDATE Est3_importe_total SET importe_total=v_importe_total WHERE ao=p_ao; 
  END IF;
      
  -- terminado ok
   RSP:= 'OK_EST3_IMPORTE_TOTAL';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST3_IMPORTE_TOTAL', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST3_IMPORTE_TOTAL: ' || RSP;
        SP_ALTA_LOG('SP_EST3_IMPORTE_TOTAL', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR SP_EST3_IMPORTE_TOTAL: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST3_IMPORTE_TOTAL', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
END SP_EST3_IMPORTE_TOTAL;
/
