CREATE OR REPLACE PROCEDURE TFC.SP_EST4_PORCENTAJE_COMPLETADO (
   p_ao                   IN EST4_PORCENTAJE_COMPLETADO.ao%TYPE, 
   RSP                     OUT VARCHAR2)
   
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST4_PORCENTAJE_COMPLETADO
   Obejtivo:               actualiza la tabla EST4_PORCENTAJE_COMPLETADO
                   
   Parmetros de entrada:  p_ao      

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
E_CONOCIDA           EXCEPTION; 
v_existe_registro    NUMBER;
v_incompleto         NUMBER(12,4);
v_completo           NUMBER(12,4);
v_porcentaje         NUMBER(12,4);

BEGIN
   v_parametros_entrada:='p_ao: ' || to_char(p_ao);
   
  -- Comprueba si los parmetros de entrada son nulos
   IF (p_ao IS NULL) THEN 
     RSP:='El campo p_ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

  --Calculamos el nmero de regalos que han quedado en estado "NO_FONS_SUFICIENT"
  SELECT COUNT(*) INTO v_incompleto FROM Regala WHERE  EXTRACT (YEAR FROM FECHA_FINAL_RECAUDACIN)=p_ao
  AND ESTADO_REGALO='NO_SUFICIENTS_FONS';
  
  --Calculamos el nmero de regalos que estn en cualquier estado que de ha entender 
  --que se han conseguido los fondos suficientes, que son: "PAGAMENT_COMPLETAT",
  --"ENVIAT" y "REBUT".
  SELECT COUNT(*) INTO v_completo FROM Regala WHERE  EXTRACT (YEAR FROM FECHA_FINAL_RECAUDACIN)=p_ao
  AND ESTADO_REGALO='PAGAMENT_COMPLETAT' OR ESTADO_REGALO='ENVIAT' OR ESTADO_REGALO='REBUT';
  
  --Comprobamos si existe el registro en la tabla
  SELECT COUNT(*) INTO v_existe_registro FROM Est4_porcentaje_completado WHERE ao=p_ao;
   
  IF v_existe_registro=0 THEN
     IF v_completo!=0 AND v_incompleto!=0 THEN
       v_porcentaje:=(v_incompleto/v_completo)*100;
     ELSE
       v_porcentaje:=0;
     END IF;
     INSERT INTO Est4_porcentaje_completado VALUES (p_ao, v_porcentaje);
  END IF;
  
  IF v_existe_registro!=0 THEN
    IF v_completo!=0 AND v_incompleto!=0 THEN
       v_porcentaje:=(v_incompleto/v_completo)*100;
     ELSE
       v_porcentaje:=0;
     END IF;
  UPDATE Est4_porcentaje_completado SET porcentaje=v_porcentaje WHERE ao=p_ao; 
  END IF;
      
  -- terminado ok
   RSP:= 'OK_EST4_PORCENTAJE_COMPLETADO';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST4_PORCENTAJE_COMPLETADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST4_PORCENTAJE_COMPLETADO: ' || RSP;
        SP_ALTA_LOG('SP_EST4_PORCENTAJE_COMPLETADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR SP_EST4_PORCENTAJE_COMPLETADO: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST4_PORCENTAJE_COMPLETADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
END SP_EST4_PORCENTAJE_COMPLETADO;
/
