CREATE OR REPLACE PROCEDURE TFC.SP_EST6_MAS_HOMENAJEADO (
   p_homenajeado           IN Miembro.Id_miembro%TYPE,
   p_fecha                 IN Transaccin.fecha%TYPE,  
   RSP                     OUT VARCHAR2)
   
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST6_MAS_HOMENAJEADO
   Obejtivo:               actualiza la tabla EST6_MAS_HOMENAJEADO
                   
   Parmetros de entrada:  p_homenajeado, p_fecha           

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_existe_registro    INTEGER;
E_CONOCIDA           EXCEPTION; 
v_integridad         NUMBER;
p_ao                NUMBER(4);
P_mes                NUMBER(2);
v_n_veces            NUMBER;

BEGIN
   v_parametros_entrada:='p_homenajeado: '|| to_char(p_homenajeado)
                      || 'p_fecha: '      || to_char(p_fecha);

  -- Comprueba si los parmetros de entrada son nulos
   IF (p_homenajeado IS NULL) THEN 
     RSP:='El campo p_producto no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
   
   IF (p_fecha IS NULL) THEN 
     RSP:='El campo p_fecha no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

  -- Extraemos el ao de la fecha
  p_ao:= TO_CHAR(p_fecha,'yyyy');
  -- Extraemos el mes de la fecha
  p_mes:= TO_CHAR(p_fecha,'MM');
  
  --Obtenemos el nmero de regalos que ha recibido un miembro en un fecha
  SELECT COUNT(*) INTO V_INTEGRIDAD FROM 
 (SELECT HOMENAJEADO FROM DESEA D JOIN REGALA R
  ON D.ID_DESEO=R.DESEO AND EXTRACT (YEAR FROM R.FECHA_FINAL_RECAUDACIN)=p_ao AND 
                               EXTRACT (MONTH FROM R.FECHA_FINAL_RECAUDACIN)=p_mes AND 
                               D.HOMENAJEADO=p_homenajeado AND R.ESTADO_REGALO='PAGAMENT_COMPLETAT');
                               
  --Comprobamos si existe datos para un ao y un mes concretos
  SELECT COUNT(*) INTO v_existe_registro FROM EST6_MAS_HOMENAJEADO WHERE ao=p_ao AND mes=p_mes;
  
  --Si no existe el registro para el par ao-mes se gener un nuevo mximo
  IF v_existe_registro=0 THEN 
    INSERT INTO EST6_MAS_HOMENAJEADO(AO,MES,ID_MIEMBRO,N_VECES) VALUES (p_ao,p_mes,p_homenajeado,v_integridad+1);
  END IF;
  
  --Si existe el registro para el par ao-mes debemos determinar cual es el mximo
  IF v_existe_registro!=0 THEN 
    SELECT MAX(n_veces) INTO v_n_veces FROM EST6_MAS_HOMENAJEADO WHERE ao=p_ao AND mes=p_mes AND ROWNUM=1;
      IF v_n_veces<v_integridad THEN
        DELETE FROM EST6_MAS_HOMENAJEADO WHERE ao=p_ao AND mes=p_mes;
        INSERT INTO EST6_MAS_HOMENAJEADO(AO,MES,ID_MIEMBRO,N_VECES) VALUES (p_ao,p_mes,p_homenajeado,v_integridad);
    END IF;
    --Si se dan varios valores mximos de forma simultanea
    IF v_n_veces=v_integridad THEN
      INSERT INTO EST6_MAS_HOMENAJEADO(AO,MES,ID_MIEMBRO,N_VECES) VALUES (p_ao,p_mes,p_homenajeado,v_integridad);
    END IF;
  END IF; 
  -- terminado ok
   RSP:= 'OK_EST6_MAS_HOMENAJEADO';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST6_MAS_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST6_MAS_HOMENAJEADO: ' || RSP;
        SP_ALTA_LOG('SP_EST6_MAS_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST6_MAS_HOMENAJEADO: ' || v_err_msg;
        SP_ALTA_LOG('EST6_MAS_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST6_MAS_HOMENAJEADO;
/
