CREATE OR REPLACE PROCEDURE TFC.SP_EST8_MAS_PARTICIPATIVO (
   p_fecha                 IN Transaccin.fecha%TYPE,  
   RSP                     OUT VARCHAR2)
   
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST8_MAS_PARTICIPATIVO
   Obejtivo:               actualiza la tabla EST8_MAS_PARTICIPATIVO
                   
   Parmetros de entrada:  p_fecha           

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_existe_registro    INTEGER;
E_CONOCIDA           EXCEPTION; 
p_ao                NUMBER(4);
p_mes                NUMBER(2);
v_aportador          NUMBER;
v_participacion      NUMBER;
v_id_miembro         NUMBER;
v_cantidad_maxima    NUMBER;

CURSOR cur IS SELECT APORTADOR,COUNT (DISTINCT REGALO) AS PARTICIPACION FROM TRANSACCIN WHERE 
  EXTRACT (YEAR FROM FECHA)=p_ao AND 
  EXTRACT (MONTH FROM FECHA)=p_mes AND
  TIPO='PAGO'
  GROUP BY APORTADOR ORDER BY APORTADOR ASC;

BEGIN
   v_parametros_entrada:='p_fecha: '|| to_char(p_fecha);
   
  -- Comprueba si los parmetros de entrada son nulos
   IF (p_fecha IS NULL) THEN 
     RSP:='El campo p_fecha no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

  -- Extraemos el ao de la fecha
  p_ao:= TO_CHAR(p_fecha,'yyyy');
  -- Extraemos el mes de la fecha
  p_mes:= TO_CHAR(p_fecha,'MM');
  
  --Obtenemos el maximo que gasta un miembro dado un ao y un mes
  SELECT * INTO v_id_miembro,v_cantidad_maxima FROM
  (SELECT APORTADOR,COUNT (DISTINCT REGALO) AS PARTICIPACION FROM TRANSACCIN WHERE 
  EXTRACT (YEAR FROM FECHA)=p_ao AND 
  EXTRACT (MONTH FROM FECHA)=p_mes
  GROUP BY APORTADOR ORDER BY APORTADOR DESC)WHERE ROWNUM=1;
  
  --Comprobamos si existe datos para un ao y un mes concretos
  SELECT COUNT(*) INTO v_existe_registro FROM EST8_MAS_PARTICIPATIVO WHERE ao=p_ao AND mes=p_mes;
  
  --Si no existe el registro para el par ao-mes se gener un nuevo mximo
  IF v_existe_registro=0 THEN 
     --Insertamos el nuevo maximo o maximos
   OPEN CUR;
      LOOP 
       FETCH CUR INTO V_APORTADOR,V_PARTICIPACION;             
       EXIT WHEN CUR%NOTFOUND;
           --Determinamos si para un mismo par ao-mes hay varios mximos
           IF V_PARTICIPACION>=V_CANTIDAD_MAXIMA THEN
             INSERT INTO EST8_MAS_PARTICIPATIVO(AO,MES,ID_MIEMBRO,N_VECES) VALUES (p_ao,p_mes,v_APORTADOR,v_PARTICIPACION); 
           END IF;
      END LOOP;
     CLOSE cur;
   END IF;
  
  --Si existe el registro para el par ao-mes debemos actualizamos la tabla
  IF v_existe_registro!=0 THEN 
     --Borramos los registros 
     DELETE FROM EST8_MAS_PARTICIPATIVO WHERE ao=p_ao AND mes=p_mes;
     --Insertamos el nuevo maximo o maximos
     OPEN CUR;
      LOOP 
       FETCH CUR INTO V_APORTADOR,V_PARTICIPACION;           
       EXIT WHEN CUR%NOTFOUND;
          IF V_PARTICIPACION>=V_CANTIDAD_MAXIMA THEN
             INSERT INTO EST8_MAS_PARTICIPATIVO(AO,MES,ID_MIEMBRO,N_VECES) VALUES (p_ao,p_mes,v_APORTADOR,v_PARTICIPACION);    
           END IF;   
      END LOOP;
     CLOSE cur;
  END IF;
  --terminado ok
   RSP:= 'OK_EST8_MAS_PARTICIPATIVO';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST8_MAS_PARTICIPATIVO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST8_MAS_PARTICIPATIVO: ' || RSP;
        SP_ALTA_LOG('SP_EST8_MAS_PARTICIPATIVO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST8_MAS_PARTICIPATIVO: ' || v_err_msg;
        SP_ALTA_LOG('EST8_MAS_PARTICIPATIVO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST8_MAS_PARTICIPATIVO;
/
