CREATE OR REPLACE PROCEDURE TFC.SP_EST9_GRUPO_MAS_GASTADOR (
   p_fecha                 IN Transaccin.fecha%TYPE,  
   RSP                     OUT VARCHAR2)
   
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST9_GRUPO_MAS_GASTADOR
   Obejtivo:               actualiza la tabla EST9_GRUPO_MAS_GASTADOR
                   
   Parmetros de entrada:  p_fecha           

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_existe_registro    INTEGER;
E_CONOCIDA           EXCEPTION; 
p_ao                NUMBER(4);
p_mes                NUMBER(2);
v_grupo              NUMBER;
v_total              NUMBER;
v_id_grupo           NUMBER;
v_maximo             NUMBER;

CURSOR cur IS SELECT R.GRUPO, SUM(T.CANTIDAD) AS TOTAL FROM TRANSACCIN T 
  JOIN REGALA R ON T.REGALO=R.ID_REGALO AND 
  EXTRACT (YEAR FROM T.FECHA)=p_ao AND 
  EXTRACT (MONTH FROM T.FECHA)=p_mes
  GROUP BY R.GRUPO ORDER BY TOTAL DESC;

BEGIN
   v_parametros_entrada:='p_fecha: '|| to_char(p_fecha);
   
  -- Comprueba si los parmetros de entrada son nulos
   IF (p_fecha IS NULL) THEN 
     RSP:='El campo p_fecha no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

  -- Extraemos el ao de la fecha
  p_ao:= TO_CHAR(p_fecha,'yyyy');
  -- Extraemos el mes de la fecha
  p_mes:= TO_CHAR(p_fecha,'MM');
  
  --Obtenemos el mximo que gasta un grupo dado un ao y un mes
  SELECT * INTO V_ID_GRUPO, V_MAXIMO FROM
  (SELECT R.GRUPO, SUM(T.CANTIDAD) AS TOTAL FROM TRANSACCIN T 
  JOIN REGALA R ON T.REGALO=R.ID_REGALO AND 
  EXTRACT (YEAR FROM T.FECHA)=p_ao AND 
  EXTRACT (MONTH FROM T.FECHA)=p_mes
  GROUP BY R.GRUPO ORDER BY TOTAL DESC)WHERE ROWNUM=1;
  
  --Comprobamos si existe datos para un ao y un mes concretos
  SELECT COUNT(*) INTO v_existe_registro FROM EST9_GRUPO_MAS_GASTADOR WHERE ao=p_ao AND mes=p_mes;
  
  --Si no existe el registro para el par ao-mes se gener un nuevo mximo
  IF v_existe_registro=0 THEN 
     --Insertamos el nuevo maximo o maximos
   OPEN CUR;
      LOOP 
       FETCH CUR INTO V_GRUPO,V_TOTAL;             
       EXIT WHEN CUR%NOTFOUND;
           --Determinamos si para un mismo par ao-mes hay varios mximos
           IF V_TOTAL>=V_MAXIMO THEN
             INSERT INTO EST9_GRUPO_MAS_GASTADOR(AO,MES,ID_GRUPO,TOTAL) VALUES (p_ao,p_mes,V_GRUPO,V_TOTAL); 
           END IF;
      END LOOP;
     CLOSE cur;
   END IF;
  
  --Si existe el registro para el par ao-mes debemos actualizamos la tabla
  IF v_existe_registro!=0 THEN 
     --Borramos los registros 
     DELETE FROM EST9_GRUPO_MAS_GASTADOR WHERE ao=p_ao AND mes=p_mes;
     --Insertamos el nuevo maximo o maximos
     OPEN CUR;
      LOOP 
       FETCH CUR INTO V_GRUPO,V_TOTAL;           
       EXIT WHEN CUR%NOTFOUND;
          IF V_TOTAL>=V_MAXIMO THEN
            INSERT INTO EST9_GRUPO_MAS_GASTADOR(AO,MES,ID_GRUPO,TOTAL) VALUES (p_ao,p_mes,V_GRUPO,V_TOTAL); 
          END IF;   
      END LOOP;
     CLOSE cur;
  END IF;
  --terminado ok
   RSP:= 'OK_EST9_GRUPO_MAS_GASTADOR';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST9_GRUPO_MAS_GASTADOR', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST9_GRUPO_MAS_GASTADOR: ' || RSP;
        SP_ALTA_LOG('SP_EST9_GRUPO_MAS_GASTADOR', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR SP_EST9_GRUPO_MAS_GASTADOR: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST9_GRUPO_MAS_GASTADOR', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST9_GRUPO_MAS_GASTADOR;
/
