CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_GRUPO (
p_dedicatoria_grupal        IN GRUPO.dedicatoria_grupal%type,
p_homenajeado               IN GRUPO.homenajeado%type,
p_receptor                  IN GRUPO.receptor%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_GRUPO
   Obejtivo:               Da de alta un grupo
                   
   Parmetros de entrada:  p_dedicatoria_grupal, p_homenajeado, p_grupo                           

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;
v_receptor           NUMBER;

BEGIN
   v_parametros_entrada:= 'Dedicatoria grupal: ' || p_dedicatoria_grupal
                       || ' ,p_homenajeado: '       || to_char(p_homenajeado)
                       || ' ,p_receptor: '          || to_char(p_receptor); 
                     
   -- Comprueba si alguno de los parmetros obligatorios es nulo
    IF (p_dedicatoria_grupal IS NULL) THEN 
      RSP:= 'Se debe indicar una dedicatora grupal' ;
      RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba que si se indica el homenajeado este tenga algn deseo
   IF p_homenajeado!=0 THEN
   IF (p_homenajeado IS NOT NULL) THEN
    SELECT COUNT(*) into v_integridad FROM Desea WHERE homenajeado=p_homenajeado;
     IF (v_integridad=0) THEN
      RSP:='El homenajeado no desea ningn regalo';
      RAISE E_CONOCIDA;
     END IF;
   END IF; 
   END IF;
   
   --Comprueba que si se indica el receptor este sea miembro de la red
   IF p_receptor!=0 THEN
   IF (p_receptor IS NOT NULL) THEN
    SELECT COUNT(*) INTO v_integridad FROM Miembro WHERE id_miembro=p_receptor;
     IF (v_integridad=0) THEN
      RSP:='El receptor no es miembro de la red social';
      RAISE E_CONOCIDA;
     END IF;
   END IF; 
   END IF;
   
   --En caso de no designar al receptor, pero si al homenajeado, 
   --este ltimo ser automaticamente receptor.
   IF (p_receptor) IS NULL AND (p_homenajeado) IS NOT NULL THEN
     INSERT INTO Grupo (dedicatoria_grupal, homenajeado, receptor) 
     VALUES  (p_dedicatoria_grupal,p_homenajeado,p_homenajeado);
   ELSE
    -- Inserta los datos del grupo
     INSERT INTO Grupo (dedicatoria_grupal, homenajeado, receptor) 
     VALUES  (p_dedicatoria_grupal,p_homenajeado,p_receptor);
   END IF;
   
   UPDATE Grupo SET homenajeado=0 WHERE homenajeado IS NULL; 
   
   RSP:='OK_ALTA_GRUPO';
   SP_ALTA_LOG('SP_ALTA GRUPO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_GRUPO: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_GRUPO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_GRUPO: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_GRUPO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_ALTA_GRUPO;
/
