CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_GRUPO (
p_id_grupo          IN GRUPO.id_grupo%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_GRUPO
   Obejtivo:               Da de baja un grupo
                   
   Parmetros de entrada:  p_id_grupo

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'id_grupo: '       || p_id_grupo; 
                     
   --Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_id_grupo IS NULL) THEN 
      RSP:= 'El campo grupo no puede ser nulo';
      RAISE E_CONOCIDA;
   END IF;
   
   --Comprueba que el grupo no tiene regalos
   SELECT COUNT(*) INTO v_integridad FROM Regala WHERE grupo=p_id_grupo;
   IF (v_integridad !=0) THEN
       RSP:= 'El grupo tiene regalos, deben darse de baja previamente';
       RAISE E_CONOCIDA;
   END IF;
   
   --Comprueba que el grupo no tiene participantes
   SELECT COUNT(*) INTO v_integridad FROM Agrupa WHERE grupo=p_id_grupo;
   IF (v_integridad !=0) THEN
       RSP:= 'El grupo tiene participantes, deben darse de baja previamente';
       RAISE E_CONOCIDA;
   END IF;
   

   --Borra los datos del grupo
   DELETE FROM Grupo WHERE id_grupo=p_id_grupo;
   RSP:='OK_BAJA_GRUPO';
   SP_ALTA_LOG('SP_BAJA_GRUPO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_GRUPO: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_GRUPO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_GRUPO: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_GRUPO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_BAJA_GRUPO;
/
