create or replace PROCEDURE     SP_ALTA_REGALO (
p_grupo                     IN REGALA.grupo%type,
p_deseo                     IN REGALA.deseo%type,
p_fecha_final_recaudacin   IN REGALA.fecha_final_recaudacin%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_REGALO
   Obejtivo:               Da de alta un regalo en un grupo
                   
   Parmetros de entrada:  p_grupo, p_deseo,p_fecha_final_recaudacin
                           
   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;
v_homenajeado        NUMBER;
v_homenajeado2       NUMBER;
homenajeado2         NUMBER;
regalo2              NUMBER;
v_estado             VARCHAR2(30);
v_hoy                DATE:=SYSDATE;

BEGIN
   v_parametros_entrada:= 'p_grupo: '                    || p_grupo 
                       || ' ,p_deseo: '                    || to_char(p_deseo)
                       || ' ,p_fecha_final_recaudacin: '  || to_char(p_fecha_final_recaudacin);
                                 
   --Comprueba si alguno de los parmetros obligatorios es nulo
    IF (p_grupo IS NULL) THEN 
      RSP:= 'El campo grupo no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_deseo IS NULL) THEN 
      RSP:= 'El campo deseo no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_fecha_final_recaudacin IS NULL) THEN 
      RSP:= 'El campo fecha_final_recaudacin no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
  --Comprueba que el grupo est disponible
  IF (p_grupo IS NOT NULL) THEN 
      SELECT COUNT(*) INTO v_integridad FROM Grupo WHERE id_grupo=p_grupo;
      IF (v_integridad=0) THEN
         RSP:= 'El grupo no est disponible';
         RAISE E_CONOCIDA;
      END IF;
   END IF;
   
  --Comprueba que el regalo est disponible
   IF (p_deseo IS NOT NULL) THEN 
      select count(*) into v_integridad from Desea where id_deseo=p_deseo;
      IF (v_integridad=0) THEN
         RSP:= 'El deseo no est disponible';
         RAISE E_CONOCIDA;
      END IF;
   END IF;
   
   --Comprueba que el grupo tiene homenajeado
   SELECT homenajeado INTO v_homenajeado FROM Grupo WHERE id_grupo=p_grupo;
   IF (v_homenajeado=0) THEN
         RSP:= 'El grupo no tiene homenajeado. Debe darse de alta previamente';
         RAISE E_CONOCIDA;
   END IF;

   --Comprueba si ese regalo ya est adjudicado al grupo
   select count(*) into v_integridad from Regala where deseo=p_deseo AND grupo=p_grupo;
   IF (v_integridad!=0) THEN
      RSP:= 'Ya existe ese regalo para ese grupo';
      RAISE E_CONOCIDA;
   END IF;
   
   --Comprueba que el homenajeado del grupo y el solicitante del deseo son la misma persona
    SELECT homenajeado INTO v_homenajeado FROM Grupo WHERE id_grupo=p_grupo;
    SELECT homenajeado INTO v_homenajeado2 FROM Desea WHERE id_deseo=p_deseo;
    IF (v_homenajeado!=v_homenajeado2) THEN
         RSP:= 'El homenajeado del grupo y el solicitante del deseo deben ser la misma persona';
         RAISE E_CONOCIDA;
   END IF;

  --Un regalo recien aadido tiene por defecto el estado 'AFEGIT_A_UN_GRUP'
   v_estado:='AFEGIT_A_UN_GRUP';
  
-- Inserta los datos del miembro y el su deseo
   INSERT INTO Regala (grupo,deseo,fecha_inicio_recaudacin, 
                       fecha_final_recaudacin, estado_regalo) 
   VALUES  (p_grupo,p_deseo,v_hoy,
            p_fecha_final_recaudacin,v_estado);
            
   RSP:='OK_ALTA_REGALO';
   SP_ALTA_LOG('SP_ALTA_REGALO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_REGALO: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_REGALO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_REGALO: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_REGALO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_ALTA_REGALO;