CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_REGALO (
p_id_regalo          IN REGALA.id_regalo%type,

RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_REGALO
   Obejtivo:               Da de alta un regalo en un grupo
                   
   Parmetros de entrada:  p_id_regalo

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;
estado2              VARCHAR2 (40);

CURSOR cur IS SELECT id_transaccin,regalo,aportador,cantidad,tipo,traza_abono,fecha,forma_pago,visible 
FROM transaccin WHERE regalo=p_id_regalo ORDER BY fecha;
---UNabono                VARCHAR2 (2000);

v_abono              NUMBER;
v_id_transaccin     Transaccin.id_transaccin%type;
v_regalo             Transaccin.regalo%type;
v_aportador          Transaccin.aportador%type;
v_cantidad           Transaccin.cantidad%type;
v_tipo               Transaccin.tipo%type;
v_traza_abono        Transaccin.traza_abono%type;
v_fecha              Transaccin.fecha%type;
v_forma_pago         Transaccin.forma_pago%type;
v_visible            Transaccin.visible%type;
v_hoy                DATE:=SYSDATE;

BEGIN
   v_parametros_entrada:= 'p_id_regalo: '    || p_id_regalo; 
                     
   -- Comprueba si alguno de los parmetros obligatorios es nulo
    IF (p_id_regalo IS NULL) THEN 
      RSP:= 'El campo regalo no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
  -- Comprueba que el regalo esta disponible
      SELECT COUNT(*) INTO v_integridad FROM Regala WHERE id_regalo=p_id_regalo;
      IF (v_integridad=0) THEN
         RSP:= 'El regalo no est existe. No se puede dar de baja';
         RAISE E_CONOCIDA;
      END IF;
   
  -- Comprueba que el regalo esta en estado "AFEGIT_A_UN_GRUP"
  SELECT estado_regalo INTO estado2 FROM Regala WHERE id_regalo=p_id_regalo;   
   IF (estado2!='AFEGIT_A_UN_GRUP') THEN
         RSP:= 'El regalo no se puede dar de baja. No es compatible su estado con la baja';
         RAISE E_CONOCIDA;
   END IF;
   
  -- Comprueba que el regalo tiene aportaciones y efectua los abonos 
  SELECT COUNT(*) INTO v_integridad FROM Transaccin WHERE regalo=p_id_regalo;
  IF (v_integridad!=0) THEN
     --Efectuamos un recorrido por transacciones y realizamos los abonos
     OPEN CUR;
     LOOP 
      FETCH CUR INTO v_id_transaccin,v_regalo,v_aportador,v_cantidad,v_tipo,v_traza_abono,v_fecha,v_forma_pago,v_visible;
      
      v_cantidad:=-v_cantidad;
      v_tipo:='ABONO';
      v_traza_abono:=v_id_transaccin;
      
                              
      EXIT WHEN CUR%NOTFOUND;
      insert into Transaccin(regalo,aportador,cantidad,tipo,traza_abono,fecha,forma_pago,visible) 
      values(v_regalo,v_aportador,v_cantidad,v_tipo,v_traza_abono,v_hoy,v_forma_pago,v_visible);
      
     END LOOP;
     CLOSE cur;
  END IF;
    
 -- Modifica el estado del regalo
   UPDATE Regala SET estado_regalo='ELIMINAT' WHERE id_regalo=p_id_regalo;
  
   RSP:='OK_BAJA_REGALO';
   SP_ALTA_LOG('SP_BAJA_REGALO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_REGALO: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_REGALO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_REGALO: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_REGALO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_BAJA_REGALO;
/
