CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_PARTICIPANTE (
p_id_miembro           IN MIEMBRO.id_miembro%type,
p_grupo                IN AGRUPA.grupo%type,
RSP                    OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_PARTICIPANTE
   Obejtivo:               Da de baja un participante en un grupo
                   
   Parmetros de entrada:  p_id_miembro,p_grupo           

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;
p_email              VARCHAR2(40);

BEGIN
   v_parametros_entrada:= 'p_id_miembro: '             || (p_id_miembro)
                      || ' ,p_grupo: '                  || to_char(p_grupo);

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_id_miembro IS NULL) THEN 
     RSP:= 'El campo miembro no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
  
   IF (p_grupo IS NULL) THEN 
     RSP:= 'El campo grupo no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
    SELECT email INTO p_email FROM Miembro WHERE id_miembro=p_id_miembro ;
    
   -- Comprueba si el miembro ya esta asignado al grupo   
   SELECT COUNT(*) INTO v_integridad FROM Agrupa WHERE email=p_email AND grupo=p_grupo;
   IF (v_integridad=0) THEN
     RSP:= 'El miembro no esta asociado al grupo';
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si el miembro tiene aportaciones en el grupo
  SELECT COUNT(*) INTO v_integridad FROM Transaccin WHERE aportador=p_id_miembro;
   IF (v_integridad!=0) THEN
     RSP:= 'El miembro no puede darse de baja. Anule las aportaciones';
     RAISE E_CONOCIDA;
   END IF;
   
   -- Inserta los datos del miembro en el grupo:  
   DELETE FROM Agrupa WHERE p_email=email and p_grupo=grupo;
   RSP:='OK_BAJA_PARTICIPANTE';
   SP_ALTA_LOG('SP_BAJA_PARTICIPANTE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;

   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_PARTICIPANTE: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_PARTICIPANTE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_PARTICIPANTE: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_PARTICIPANTE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_BAJA_PARTICIPANTE;
/
