CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_HOMENAJEADO (
p_homenajeado       IN GRUPO.homenajeado%type,
p_grupo             IN GRUPO.id_grupo%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_HOMENAJEADO
   Obejtivo:               Da de alta un homenajeado en un grupo
                   
   Parmetros de entrada:  p_homenajeado,p_grupo

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;
v_homenajeado        NUMBER;
v_receptor           NUMBER;

BEGIN
   v_parametros_entrada:= 'p_homenajeado: '    || p_homenajeado 
                       || ' ,p_grupo: '        || to_char(p_grupo);
                     
   -- Comprueba si alguno de los parmetros obligatorios es nulo
    IF (p_homenajeado IS NULL) THEN 
      RSP:= 'El campo homenajeado no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_grupo IS NULL) THEN 
      RSP:= 'El campo grupo no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
  -- Comprueba la integridad de los identificadores
  IF (p_homenajeado IS NOT NULL) THEN 
      SELECT COUNT(*) into v_integridad FROM Desea WHERE homenajeado=p_homenajeado;
      IF (v_integridad=0) THEN
         RSP:= 'El homenajeado no tiene deseos y no puede darse de alta';
         RAISE E_CONOCIDA;
      END IF;
   END IF;
  
   IF (p_grupo IS NOT NULL) THEN 
      SELECT COUNT(*) INTO v_integridad FROM Grupo WHERE id_grupo=p_grupo;
      IF (v_integridad=0) THEN
         RSP:= 'El grupo no existe. Debe de darse de alta previamente';
         RAISE E_CONOCIDA;
      END IF;
   END IF;
   
 --Comprueba si el grupo tiene homenajeado
   SELECT homenajeado INTO v_homenajeado FROM Grupo WHERE id_grupo=p_grupo;
   IF (v_homenajeado!=0) THEN
      RSP:= 'El grupo ya tiene homenajeado. Para cambiarlo debe de dar de baja el actual';
         RAISE E_CONOCIDA;
   END IF;
   
 -- Inserta al homenajeado en el grupo
   UPDATE Grupo SET homenajeado=p_homenajeado WHERE id_grupo=p_grupo;
   
 --Si no hay receptor en el grupo este ser el homenajeado
   SELECT receptor INTO v_receptor FROM Grupo WHERE id_grupo=p_grupo;
   IF (v_receptor=0) THEN
      UPDATE Grupo SET receptor=p_homenajeado WHERE id_grupo=p_grupo;   
   END IF;

   RSP:='OK_ALTA HOMENAJEADO';
   SP_ALTA_LOG('SP_ALTA_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_HOMENAJEADO: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_HOMENAJEADO: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_ALTA_HOMENAJEADO;
/
