CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_HOMENAJEADO (
p_homenajeado       IN GRUPO.homenajeado%type,
p_grupo             IN GRUPO.id_grupo%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_HOMENAJEADO
   Obejtivo:               Da de baja un homenajeado en un grupo
                   
   Parmetros de entrada:  p_homenajeado,p_grupo

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_homenajeado: '    || p_homenajeado 
                       || ' ,p_grupo: '        || to_char(p_grupo);
                     
   -- Comprueba si alguno de los parmetros obligatorios es nulo
    IF (p_homenajeado IS NULL) THEN 
      RSP:= 'El campo homenajeado no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_grupo IS NULL) THEN 
      RSP:= 'El campo grupo no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
  
  -- Comprueba que la correlacin homenajeado con grupo
  IF (p_homenajeado IS NOT NULL) THEN 
      SELECT COUNT(*) INTO v_integridad FROM Grupo WHERE homenajeado=p_homenajeado AND id_grupo=p_grupo;
      IF (v_integridad=0) THEN
         RSP:= 'El homenajeado no esta asignado al grupo';
         RAISE E_CONOCIDA;
      END IF;
   END IF;
   
   --Comprueba que el homenajeado no tiene deseos en proceso de regalo
  SELECT COUNT(*) INTO v_integridad FROM Regala WHERE grupo=p_grupo;  
  IF (v_integridad!=0) THEN
   RSP:= 'El homenajeado tiene deseos que se estn procesando como regalos';
         RAISE E_CONOCIDA;
  END IF;
  
  -- Borra al homenajeado en el grupo
   UPDATE Grupo SET homenajeado=0 WHERE id_grupo=p_grupo;
   RSP:='OK_BAJA_HOMENAJEADO';
   SP_ALTA_LOG('SP_BAJA_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_HOMENAJEADO: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_HOMENAJEADO: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_HOMENAJEADO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_BAJA_HOMENAJEADO;
/
