CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_TRANSACCIN(
p_regalo          IN Transaccin.regalo%type,
p_aportador       IN Transaccin.aportador%type,
p_cantidad        IN Transaccin.cantidad%type,
p_forma_pago      IN transaccin.forma_pago%type,
p_visible         IN transaccin.visible%type,

RSP                OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_TRANSACCION
   Obejtivo:               Da de alta una transaccin
                   
   Parmetros de entrada:  p_regalo, p_aportador, p_cantidad,p_forma_pago, p_visible

   Resultado:             'OK_ALTA_DESEO' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
v_id_grupo           INTEGER;
v_id_grupo2          INTEGER;
v_email              VARCHAR2(80);
E_CONOCIDA           EXCEPTION;
v_estado_regalo      VARCHAR2(40);
v_parcial_pago       NUMBER;
v_parcial_abono      NUMBER;
v_fecha_final_recaudacin DATE;
v_id_deseo           NUMBER;
v_id_regalo          NUMBER;
v_precio             NUMBER;
v_homenajeado        NUMBER;
v_necesario          NUMBER;
v_traza_abono        NUMBER;
v_tipo               VARCHAR2(15);
v_id_artculo        NUMBER;
v_hoy                DATE:=SYSDATE;

--Variables para el modulo estadstico
v_fecha                    DATE;
ao                        INTEGER;



BEGIN
   v_parametros_entrada:= 'p_regalo: '      || p_regalo 
                       || ' ,p_aportador: '   || to_char(p_aportador)
                       || ' ,p_cantidad: '    || to_char(p_cantidad)
                       || ' ,p_forma_pago: '  || to_char(p_forma_pago)
                       || ' ,p_visible: '     || to_char(p_visible);
                     
   -- Comprueba si alguno de los parmetros obligatorios es nulo
    IF (p_regalo IS NULL) THEN 
      RSP:= 'El campo regalo no puede ser nulo';
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_aportador IS NULL) THEN 
      RSP:= 'El campo aportador no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_cantidad IS NULL) THEN 
      RSP:= 'El campo cantidad no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_forma_pago IS NULL) THEN 
      RSP:= 'El campo forma_pago no puede ser nulo';
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_visible IS NULL) THEN 
      RSP:= 'El campo visible no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba que el regalo esta disponible
   SELECT COUNT(*) INTO v_integridad FROM Regala WHERE id_regalo=p_regalo;
   IF (v_integridad=0) THEN
      RSP:= 'El regalo no est disponible.Compruebe que ha sido aadido como regalo';
      RAISE E_CONOCIDA;
   END IF;
   
   --Comprueba que el regalo esta en esta estado AFEGIT_A_UN_GRUP
   SELECT COUNT(*) INTO v_integridad FROM Regala WHERE id_regalo=p_regalo AND estado_regalo='AFEGIT_A_UN_GRUP';
   IF (v_integridad=0) THEN
      RSP:= 'El regalo no permite aportaciones.Su estado es incompatible con una aportacin';
      RAISE E_CONOCIDA;
   END IF;
   
   --Comprobamos que el aportador es miembro de la red social
   SELECT COUNT(*) INTO v_integridad FROM Miembro WHERE id_miembro=p_aportador;
   IF (v_integridad=0) THEN
      RSP:= 'El aportador no es miembro de la red social';
      RAISE E_CONOCIDA;
   END IF;
   
   --Comprueba que el miembro pertence a algn grupo
   SELECT email INTO v_email FROM Miembro WHERE id_miembro=p_aportador;
   SELECT COUNT(*) INTO v_integridad FROM Agrupa WHERE email=v_email;
   IF (v_integridad=0) THEN
      RSP:= 'El aportador no pertenece a ningun grupo';
      RAISE E_CONOCIDA;
   END IF;
   
   --Comprobamos que el aportador es miembro del grupo que selecciono el regalo
  
   --Guardamos el id de grupo que corresponde al regalo.
   SELECT grupo INTO v_id_grupo FROM Regala WHERE id_regalo=p_regalo;
   
   SELECT COUNT(*) INTO v_integridad FROM Agrupa WHERE email=v_email AND grupo=v_id_grupo;
   IF (v_integridad=0) THEN
      RSP:= 'El aportador no pertenece al grupo del que selecciono el regalo';
      RAISE E_CONOCIDA;
   END IF;
   
  --Comprobamos que la cantidad aportada no es 0 o negativa
  IF p_cantidad <=0 THEN
      RSP:= 'La cantidad aportada debe ser superior a 0';
      RAISE E_CONOCIDA;
   END IF;
  
  --Comprobamos que la cantidad aportada no es superior a la cantidad necesaria para completar el regalo
  
   --Calculamos la cantidad de dinero aportadas y abonadas para un regalo
  SELECT SUM(cantidad) INTO v_parcial_pago FROM Transaccin WHERE regalo=p_regalo AND tipo='PAGO';
  SELECT SUM(cantidad) INTO v_parcial_abono FROM Transaccin WHERE regalo=p_regalo AND tipo='ABONO';
   --Obtenemos el precio del regalo
  SELECT deseo INTO v_id_deseo FROM Regala WHERE id_regalo=p_regalo;
  SELECT artculo INTO v_id_artculo FROM Desea WHERE id_deseo=v_id_deseo;
  SELECT precio INTO v_precio FROM Artculo WHERE id_artculo=v_id_artculo; 
  
  --Obtenemos el homenajeado del deseo
  SELECT homenajeado INTO v_homenajeado FROM Desea WHERE id_deseo=v_id_deseo;
  
  IF v_parcial_abono IS NULL THEN
      v_parcial_abono:=0;
  END IF;
  
  IF v_parcial_pago IS NULL THEN
      v_parcial_pago:=0;
  END IF;
  
  IF (v_parcial_pago + v_parcial_abono + p_cantidad) > v_precio THEN
    v_necesario:=v_precio - (v_parcial_pago+v_parcial_abono);
        RSP:= 'La cantidad aportada es excesiva. Solo son necesarios '|| to_char(v_necesario)||' euros';
      RAISE E_CONOCIDA;
   END IF;
   
   --Comprobamos si la cantidad aportada completa el pago y cambiamos el estado del regalo
   IF (v_parcial_pago + v_parcial_abono + p_cantidad) = v_precio THEN
     UPDATE Regala SET estado_regalo='PAGAMENT_COMPLETAT' WHERE id_regalo=p_regalo;
   END IF;
   
   --Ponemos la traza de abono a 0
      v_traza_abono:=0;
      
   --Un transaccin es siempre por defecto un PAGO
      v_tipo:='PAGO';
   
   -- Inserta los datos de la transaccin
   INSERT INTO Transaccin (regalo,aportador,cantidad,tipo,traza_abono,fecha,forma_pago,visible) 
   VALUES (p_regalo,p_aportador,p_cantidad,v_tipo,v_traza_abono,v_hoy,p_forma_pago,p_visible);
   
   COMMIT;
   
   -- Actualizacin de las tablas del modulo estadstico
   
   -- Con el regalo obtenemos la fecha_final_recaudacin
    SELECT fecha_final_recaudacin INTO v_fecha_final_recaudacin FROM Regala WHERE deseo=v_id_deseo;
   
    ao:= TO_CHAR(v_hoy,'yyyy');
   
   --Slo se contabilizan en estas estadticas regalos que han recaudado todo el fondo necesario
   
   SELECT estado_regalo INTO v_estado_regalo FROM Regala WHERE id_regalo=p_regalo;
   
   IF v_estado_regalo = 'PAGAMENT_COMPLETAT' THEN
   
      SP_EST1_VECES_REGALADO (v_id_artculo,ao,RSP_LOG);
      IF (RSP_LOG!='OK_EST1_VECES_REGALADO') THEN
         RSP:='Error en el clculo de la estadstica 1: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
      
      SP_EST2_TOTAL_REGALOS(ao,RSP_LOG);
      IF (RSP_LOG!='OK_EST2_TOTAL_REGALOS') THEN
         RSP:='Error en el clculo de la estadstica 2: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
      
      SP_EST3_IMPORTE_TOTAL(v_id_artculo,ao,RSP_LOG);
      IF (RSP_LOG!='OK_EST3_IMPORTE_TOTAL') THEN
         RSP:='Error en el clculo de la estadstica 3: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
      
      SP_EST4_PORCENTAJE_COMPLETADO(ao,RSP_LOG);
      IF (RSP_LOG!='OK_EST4_PORCENTAJE_COMPLETADO') THEN
         RSP:='Error en el clculo de la estadstica 4: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
      
      SP_EST5_MEDIA_GASTO(v_id_artculo,ao,RSP_LOG);
      IF (RSP_LOG!='OK_EST5_MEDIA_GASTO') THEN
         RSP:='Error en el clculo de la estadstica 5: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
       
      SP_EST6_MAS_HOMENAJEADO(v_homenajeado,v_fecha_final_recaudacin,RSP_LOG);
      IF (RSP_LOG!='OK_EST6_MAS_HOMENAJEADO') THEN
         RSP:='Error en el clculo de la estadstica 6: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
   END IF;
   
      SP_EST7_MAS_GASTADOR(v_hoy,RSP_LOG);
      IF (RSP_LOG!='OK_EST7_MAS_GASTADOR') THEN
         RSP:='Error en el clculo de la estadstica 7: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
      
      SP_EST8_MAS_PARTICIPATIVO(v_hoy,RSP_LOG);
      IF (RSP_LOG!='OK_EST8_MAS_PARTICIPATIVO') THEN
         RSP:='Error en el clculo de la estadstica 8: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
      
      SP_EST9_GRUPO_MAS_GASTADOR(v_hoy,RSP_LOG);
      IF (RSP_LOG!='OK_EST9_GRUPO_MAS_GASTADOR') THEN
         RSP:='Error en el clculo de la estadstica 9: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
      
   RSP:='OK_ALTA_TRANSACCIN';
   SP_ALTA_LOG('SP_ALTA_TRANSACCIN', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_TRANSACCIN: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_TRANSACCIN', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_TRANSACCIN: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_TRANSACCIN', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);


END SP_ALTA_TRANSACCIN;
/
