CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_TRANSACCIN (
p_id_transaccin         IN Transaccin.id_transaccin%type,
RSP                      OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_TRANSACCIN
   Obejtivo:               Da de alta una camara
                   
   Parmetros de entrada:  p_id_transaccin
                                        

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;
v_regalo             INTEGER;
v_estado_regalo      VARCHAR2(40);

v_aportador          INTEGER;
v_cantidad           INTEGER;
v_tipo               VARCHAR2(40);
v_hoy                DATE:=SYSDATE;
v_forma_pago         INTEGER;
v_visible            VARCHAR2(30);

BEGIN
   v_parametros_entrada:= 'p_id_transaccin: ' || to_char(p_id_transaccin);

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_id_transaccin IS NULL) THEN 
     RSP:= 'El campo id_transaccin no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si existe una transaccin con ese id
   SELECT COUNT(*) INTO v_integridad FROM Transaccin WHERE id_transaccin=p_id_transaccin;
   IF (v_integridad=0) THEN
     RSP:= 'No existe ninguna transaccin con ese id';
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba que no se intenta abonar una cantidad que ya fue abonada
   SELECT COUNT(*) INTO v_integridad FROM Transaccin WHERE traza_abono=p_id_transaccin;
   IF (v_integridad!=0) THEN
     RSP:= 'No se pueden dar de baja una transacci ms de una vez';
     RAISE E_CONOCIDA;
   END IF;
   
   
   --Slo se permite la devolucin de regalos en estado 'AFEGIT_A_GRUP'
   SELECT regalo INTO v_regalo FROM Transaccin WHERE id_transaccin=p_id_Transaccin;
   SELECT estado_regalo INTO v_estado_regalo FROM Regala WHERE id_regalo = v_regalo;
   
   IF v_estado_regalo!='AFEGIT_A_UN_GRUP' THEN
     RSP:= 'El estado del regalo es incompatible con la anulacin de la transaccin';
     RAISE E_CONOCIDA;
   END IF;
     
  --Recuperamos los datos de la transaccin para poder generar el abono
  --Regalo ya esta disponible en v_regalo
  SELECT aportador INTO v_aportador FROM Transaccin WHERE id_transaccin=p_id_Transaccin;
  SELECT cantidad INTO v_cantidad FROM Transaccin WHERE id_transaccin=p_id_Transaccin;
  v_cantidad:=-v_cantidad;
  v_tipo:='ABONO';
  --Fecha es la automtica del sistema
  SELECT forma_pago INTO v_forma_pago FROM Transaccin WHERE id_transaccin=p_id_Transaccin;
  SELECT visible INTO v_visible FROM Transaccin WHERE id_transaccin=p_id_Transaccin;
  
  -- Generamos el abono de lta transaccion:
  INSERT INTO Transaccin(regalo,aportador,cantidad,tipo,traza_abono,fecha,forma_pago,visible) 
      VALUES(v_regalo,v_aportador,v_cantidad,v_tipo,p_id_transaccin,v_hoy,v_forma_pago,v_visible);
  
   RSP:='OK_BAJA TRANSACCIN';
   SP_ALTA_LOG('SP_BAJA_TRANSACCIN', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;

   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_TRANSACCIN: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_TRANSACCIN', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_TRANSACCIN: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_TRANSACCIN', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_BAJA_TRANSACCIN;
/
