CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_DESEO (
p_homenajeado       IN DESEA.homenajeado%type,
p_artculo          IN DESEA.artculo%type,
p_fecha_Alta        IN DESEA.fecha_Alta%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_DESEO
   Obejtivo:               Da de alta una deseo
                   
   Parmetros de entrada:  p_Homenajeado,p_artculo,p_Fecha_Alta

   Resultado:             'OK_ALTA_DESEO' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_Homenajeado: '  || p_Homenajeado 
                       || ' ,p_artculo: '     || to_char(p_artculo)
                       || ' ,p_Fecha_Alta: '   || to_char(p_Fecha_Alta);
                     
   -- Comprueba si alguno de los parmetros obligatorios es nulo
    IF (p_Homenajeado IS NULL) THEN 
      RSP:= 'El campo homenajeado no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_artculo IS NULL) THEN 
      RSP:= 'El campo artculo no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_Fecha_Alta IS NULL) THEN 
      RSP:= 'El campo fecha de alta no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba que el deseo no este repetido
   SELECT COUNT(*) INTO v_integridad FROM Desea WHERE Homenajeado=p_Homenajeado AND artculo=p_artculo;
   IF (v_integridad!=0) THEN
      RSP:= 'El deseo est repetido para ese miembro';
      RAISE E_CONOCIDA;
   END IF;
  
  --Comprueba que el artculo est disponible
  IF (p_artculo IS NOT NULL) THEN 
      SELECT COUNT(*) INTO v_integridad FROM Artculo WHERE id_artculo=p_artculo;
      IF (v_integridad=0) THEN
         RSP:= 'El artculo no est disponible';
         RAISE E_CONOCIDA;
      END IF;
   END IF;
  
  --Comprueba que el miembro est disponible
   IF (p_Homenajeado IS NOT NULL) THEN 
      SELECT COUNT(*) INTO v_integridad FROM Miembro WHERE id_miembro=p_Homenajeado;
      IF (v_integridad=0) THEN
         RSP:= 'El homenajeado no es miembro de la red social';
         RAISE E_CONOCIDA;
      END IF;
   END IF;

-- Inserta los datos del miembro y el su deseo
   INSERT INTO Desea (homenajeado, artculo, fecha_alta) 
   VALUES  (p_Homenajeado,p_artculo,p_Fecha_Alta);
   RSP:='OK_ALTA_DESEO';
   SP_ALTA_LOG('SP_ALTA_DESEO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_DESEO: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_DESEO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_DESEO: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_DESEO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_ALTA_DESEO;
/
