CREATE OR REPLACE PROCEDURE TFC.SP_MODIFICACIN_DESEO (
p_id_deseo            IN Desea.id_deseo%type,
p_nuevo_regalo        IN Desea.artculo%type,

RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_CAMARA
   Obejtivo:               Da de alta una camara
                   
   Parmetros de entrada:  p_id_deseo
                                        

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_id_deseo: '       || to_char(p_id_deseo)
                        ||' ,p_nuevo_regalo: ' || to_char(p_nuevo_regalo);

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_id_deseo IS NULL) THEN 
     RSP:= 'El campo deseo no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
   IF (p_nuevo_regalo IS NULL) THEN 
     RSP:= 'El campo nuevo_regalo no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si existe el deseo
   SELECT COUNT(*) INTO v_integridad FROM Desea WHERE id_deseo=p_id_deseo;
   IF (v_integridad=0) THEN
     RSP:= 'El deseo no existe. No se puede modificar';
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba que existe el nuevo regalo
   SELECT COUNT(*) INTO v_integridad FROM Artculo WHERE id_artculo=p_nuevo_regalo;
   IF (v_integridad=0) THEN
     RSP:= 'El nuevo deseo no existe. No se puede modificar';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si el deseo se esta tramitando como regalo
   SELECT COUNT(*) INTO v_integridad FROM Regala WHERE deseo=p_id_deseo;
   IF (v_integridad!=0) THEN
     RSP:= 'El deseo se esta procesando como regalo. No se puede modificar';
     RAISE E_CONOCIDA;
   END IF;

 -- Borra los datos de la camara:
   UPDATE Desea SET artculo=p_nuevo_regalo WHERE p_id_deseo=id_deseo;
   RSP:='OK_MODIFICACIN_DESEO';
   
   SP_ALTA_LOG('SP_MODIFICACIN_DESEO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;

   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_MODIFICACIN_DESEO: ' || RSP;
        SP_ALTA_LOG('SP_MODIFICACIN_DESEO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_MODIFICACIN_DESEO: ' || v_err_msg;
        SP_ALTA_LOG('SP_MODIFICACIN_DESEO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_MODIFICACIN_DESEO;
/
