/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R5A AS OBJECT
  (
   REGALO      VARCHAR2(30),
   PRECIO      NUMBER(14,2),
   PROVEEDOR   VARCHAR2(30),
   FECHA_ALTA  VARCHAR2(10)
    );
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R5A AS TABLE OF TY_R5A;
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION ConsultaA(
  p_id_miembro IN Desea.homenajeado%type )
  
RETURN TA_R5A
IS
  PRAGMA AUTONOMOUS_TRANSACTION;
  /******************************************************************************
  Nombre:                ConsultaA
  Obejtivo:              Dado un miembro de la red social el listado de sus regalos deseados.
  Parmetros de entrada:  p_id_miembro
  Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
  'ERROR: tipo de error' si ha fallado
  ******************************************************************************/
  RSP                  VARCHAR2 (200);
  RSP_LOG              VARCHAR2 (200);
  v_parametros_entrada VARCHAR2 (200);
  v_err_msg            VARCHAR2 (200);
  v_integridad         INTEGER;
  i                    INTEGER:=1;
  resultado TA_R5A :=TA_R5A();
  REGALO               VARCHAR2(30);
  PRECIO               NUMBER(14,2);
  PROVEEDOR            VARCHAR2(30);
  FECHA_ALTA           DATE;

type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA EXCEPTION;

  BEGIN
    v_parametros_entrada:='p_id_miembro: '|| TO_CHAR(p_id_miembro);
    
    -- Comprueba si el parmetro de entrada sea nulo
    IF (p_id_miembro IS NULL) THEN
      RSP:='El campo id_miembro no puede ser nulo';
      RAISE E_CONOCIDA;
    END IF;
    
    -- Comprueba si existe el homenajeado con deseos
    SELECT COUNT(*) INTO v_integridad FROM Desea
    WHERE homenajeado = p_id_miembro;
    IF (v_integridad =0) THEN
       RSP:='No existe ninguna homenajeado con deseos con esa id';
       RAISE E_CONOCIDA;
    END IF;
    
  OPEN CUR FOR
  SELECT c.descripcin regalo, c.precio, d.nombre proveedor, a.fecha_alta
  FROM Desea a, Artculo c, Proveedor d
  WHERE a.artculo=c.id_artculo and c.proveedor=d.id_proveedor and a.homenajeado=p_id_miembro
  GROUP BY c.descripcin,c.precio, d.nombre,a.fecha_alta ORDER BY c.descripcin ASC;
  
  LOOP
    FETCH CUR INTO regalo, precio, proveedor, fecha_alta;
    EXIT
  WHEN CUR%NOTFOUND;
    resultado.extend;
    resultado(i):= TY_R5A(regalo, precio, proveedor, fecha_alta);
    i:= i+1;
  END LOOP;
  CLOSE CUR;
  
  -- terminado ok
  RSP:= 'OK';
  
  -- llamada al procedimiento que carga la tabla de log
  SP_ALTA_LOG('CONSULTA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  IF (RSP_LOG!='OK') THEN
    RSP      :='No ha sido posible dar de alta el log: '|| RSP_LOG;
    ROLLBACK;
  END IF;
  RETURN resultado;
  
EXCEPTION
WHEN E_CONOCIDA THEN
  ROLLBACK;
  RSP:= 'ERROR CONSULTAA: ' || RSP;
  SP_ALTA_LOG('CONSULTAA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
WHEN OTHERS THEN
  ROLLBACK;
  v_err_msg := SUBSTR (SQLERRM, 1, 200);
  RSP       :='ERROR CONSULTAA: ' || v_err_msg;
  SP_ALTA_LOG('CONSULTAA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
  
END ConsultaA;
/ 