/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R5B AS OBJECT
  (
   GRUPO               NUMBER(4),
   IMPORTE_TOTAL       NUMBER(14,2),
   IMPORTE_RESTANTE    NUMBER(14,2),
   PORCENTAJE          NUMBER(14,2)
    );
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R5B AS TABLE OF TY_R5B;
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION CONSULTAB
  
RETURN TA_R5B
IS
  PRAGMA AUTONOMOUS_TRANSACTION;
  /******************************************************************************
  Nombre:                ConsultaB
  Obejtivo:              Listado de todos los grupos de participantes que todava 
                         tienen algun regalo pendiente. La suma del importe total
                         de todos los regalos, el importe que queda por recoger y
                         el porcentje respecto al importe recogido.
  Parmetros de entrada: Sin parmetros
  Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
  'ERROR: tipo de error' si ha fallado
  ******************************************************************************/
  RSP                  VARCHAR2 (200);
  RSP_LOG              VARCHAR2 (200);
  v_parametros_entrada VARCHAR2 (200);
  v_err_msg            VARCHAR2 (200);
  i                    INTEGER:=1;
  resultado TA_R5B :=TA_R5B();
  V_INTEGRIDAD         NUMBER(4);
  V_GRUPO              NUMBER(4);
  V_IMPORTE_TOTAL      NUMBER(14,2);
  V_IMPORTE_PARCIAL    NUMBER(14,2);
  V_IMPORTE_RESTANTE   NUMBER(14,2);
  V_PORCENTAJE         NUMBER(14,2);
 
type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA EXCEPTION;

  BEGIN
  v_parametros_entrada:=' ';
  
  OPEN CUR FOR
  SELECT R.GRUPO AS V_GRUPO,SUM(A.PRECIO)AS V_IMPORTE_TOTAL FROM REGALA R
  JOIN DESEA D ON R.DESEO=D.ID_DESEO
  JOIN ARTCULO A ON A.ID_ARTCULO=ARTCULO
  WHERE R.ESTADO_REGALO='AFEGIT_A_UN_GRUP' GROUP BY R.GRUPO ORDER BY R.GRUPO ASC;
  
  LOOP
  FETCH CUR INTO V_GRUPO, V_IMPORTE_TOTAL;
  
  SELECT COUNT(NVL ((SUM(T.CANTIDAD)),0)) INTO V_INTEGRIDAD 
  FROM REGALA R, TRANSACCIN T 
  WHERE R.ID_REGALO=T.REGALO AND R.ESTADO_REGALO='AFEGIT_A_UN_GRUP' AND R.GRUPO=V_GRUPO 
  GROUP BY R.ID_REGALO, R.GRUPO, R.ESTADO_REGALO;
  
  IF (V_INTEGRIDAD=0) THEN
   V_IMPORTE_RESTANTE:=V_IMPORTE_TOTAL;
   V_PORCENTAJE:=0;
  ELSE
   SELECT NVL(SUM(T.CANTIDAD),0) INTO V_IMPORTE_PARCIAL FROM TRANSACCIN T
   JOIN REGALA R ON T.REGALO=R.ID_REGALO
   WHERE R.ESTADO_REGALO='AFEGIT_A_UN_GRUP' AND R.GRUPO=V_GRUPO 
   GROUP BY R.GRUPO;
     V_IMPORTE_RESTANTE:=V_IMPORTE_TOTAL-V_IMPORTE_PARCIAL;
     V_PORCENTAJE:=(V_IMPORTE_PARCIAL/V_IMPORTE_TOTAL)*100;
  END IF;
  
  EXIT WHEN CUR%NOTFOUND;
    resultado.extend;
  
    resultado(i):= TY_R5B(V_GRUPO,V_IMPORTE_TOTAL,V_IMPORTE_RESTANTE,V_PORCENTAJE);
    i:= i+1;
  END LOOP;
  CLOSE CUR;
  
  -- terminado ok
  RSP:= 'OK';
  
  -- llamada al procedimiento que carga la tabla de log
  SP_ALTA_LOG('CONSULTAB', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  IF (RSP_LOG!='OK') THEN
    RSP      :='No ha sido posible dar de alta el log: '|| RSP_LOG;
    ROLLBACK;
  END IF;
  RETURN resultado;
  
EXCEPTION
WHEN E_CONOCIDA THEN
  ROLLBACK;
  RSP:= 'ERROR CONSULTAB: ' || RSP;
  SP_ALTA_LOG('CONSULTAB', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
WHEN OTHERS THEN
  ROLLBACK;
  v_err_msg := SUBSTR (SQLERRM, 1, 200);
  RSP       :='ERROR CONSULTAB: ' || v_err_msg;
  SP_ALTA_LOG('CONSULTAB', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
  
END CONSULTAB;
/ 