/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R5C AS OBJECT
  (
   REGALO      VARCHAR2(30),
   PROVEEDOR   VARCHAR2(30)
    );
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R5C AS TABLE OF TY_R5C;
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION ConsultaC(
  p_id_categora IN Artculo.categora%type )
  
RETURN TA_R5C
IS
  PRAGMA AUTONOMOUS_TRANSACTION;
  /******************************************************************************
  Nombre:                ConsultaA
  Obejtivo:              Dado un miembro de la red social el listado de sus regalos deseados.
  Parmetros de entrada:  p_id_miembro
  Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
  'ERROR: tipo de error' si ha fallado
  ******************************************************************************/
  RSP                  VARCHAR2 (200);
  RSP_LOG              VARCHAR2 (200);
  v_parametros_entrada VARCHAR2 (200);
  v_err_msg            VARCHAR2 (200);
  i                    INTEGER:=1;
  resultado TA_R5C :=TA_R5C();
  V_INTEGRIDAD         INTEGER;
  REGALO               VARCHAR2(30);
  PROVEEDOR            VARCHAR2(30);

type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA EXCEPTION;

  BEGIN
    v_parametros_entrada:='p_id_categora: '|| TO_CHAR(p_id_categora);
    
    -- Comprueba si el parmetro de entrada sea nulo
    IF (p_id_categora IS NULL) THEN
      RSP:='El campo id_categora no puede ser nulo';
      RAISE E_CONOCIDA;
    END IF;
    
    -- Comprueba si la categora tiene artculos asociados
    SELECT COUNT(*) INTO v_integridad FROM Artculo
    WHERE categora = p_id_categora;
    IF (v_integridad =0) THEN
       RSP:='No existe ningn regalo para esa categora';
       RAISE E_CONOCIDA;
    END IF;
    
  OPEN CUR FOR
  SELECT a.descripcin regalo, b.nombre proveedor
  FROM Artculo a, Proveedor b
  WHERE a.proveedor=b.id_proveedor AND a.categora=p_id_categora
  GROUP BY a.descripcin, b.nombre;
  
  LOOP
    FETCH CUR INTO regalo, proveedor;
    EXIT
  WHEN CUR%NOTFOUND;
    resultado.extend;
    resultado(i):= TY_R5C(regalo, proveedor);
    i:= i+1;
  END LOOP;
  CLOSE CUR;
  
  -- terminado ok
  RSP:= 'OK';
  
  -- llamada al procedimiento que carga la tabla de log
  SP_ALTA_LOG('CONSULTAC', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  IF (RSP_LOG!='OK') THEN
    RSP      :='No ha sido posible dar de alta el log: '|| RSP_LOG;
    ROLLBACK;
  END IF;
  RETURN resultado;
  
EXCEPTION
WHEN E_CONOCIDA THEN
  ROLLBACK;
  RSP:= 'ERROR CONSULTAC: ' || RSP;
  SP_ALTA_LOG('CONSULTAC', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
WHEN OTHERS THEN
  ROLLBACK;
  v_err_msg := SUBSTR (SQLERRM, 1, 200);
  RSP       :='ERROR CONSULTAC: ' || v_err_msg;
  SP_ALTA_LOG('CONSULTAC', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
  
END ConsultaC;
/ 


