/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R5D AS OBJECT
  (
  ARTCULO             VARCHAR2(50),
  DESCRIPCIN          VARCHAR2(50),
  PRECIO               NUMBER,
  CATEGORA            VARCHAR2(50),
  PROVEEDOR            VARCHAR2(50)
  );
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R5D AS TABLE OF TY_R5D;
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION CONSULTAD(
  p_id_miembro IN Desea.homenajeado%type )
  
RETURN TA_R5D
IS
  PRAGMA AUTONOMOUS_TRANSACTION;
  /******************************************************************************
  Nombre:                ConsultaD
  Obejtivo:              Dado un miembro de la red social el listado detallado 
                         de todos los regalos que ha recibido.
  Parmetros de entrada:  p_id_miembro
  Resultado:             'OK' + resultado de la consulta si la ejecucin termina
                         correctamente
  'ERROR: tipo de error' si ha fallado
  ******************************************************************************/
  RSP                  VARCHAR2 (200);
  RSP_LOG              VARCHAR2 (200);
  v_parametros_entrada VARCHAR2 (200);
  v_err_msg            VARCHAR2 (200);
  v_integridad         INTEGER;
  i                    INTEGER:=1;
  resultado TA_R5D :=TA_R5D();
  ARTCULO             VARCHAR2(50);
  DESCRIPCIN          VARCHAR2(50);
  PRECIO               NUMBER;
  CATEGORA            VARCHAR2(50);
  PROVEEDOR            VARCHAR2(50);

type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA EXCEPTION;

  BEGIN
    v_parametros_entrada:='p_id_miembro: '|| TO_CHAR(p_id_miembro);
  
    -- Comprueba si el parmetro de entrada sea nulo
    IF (p_id_miembro IS NULL) THEN
      RSP:='El campo id_miembro no puede ser nulo';
      RAISE E_CONOCIDA;
    END IF;
    
    -- Comprueba si existe el homenajeado con deseos
    SELECT COUNT(*) INTO v_integridad FROM Desea
    WHERE homenajeado = p_id_miembro;
    IF (v_integridad =0) THEN
       RSP:='No existe ninguna homenajeado con deseos con esa id';
       RAISE E_CONOCIDA;
    END IF;
   
  OPEN CUR FOR
  SELECT D.ARTCULO,A.DESCRIPCIN,A.PRECIO,C.NOMBRE AS CATEGORA,P.NOMBRE AS PROVEEDOR FROM DESEA D
  JOIN REGALA R ON D.ID_DESEO=R.DESEO
  JOIN ARTCULO A ON A.ID_ARTCULO=D.ARTCULO
  JOIN CATEGORA C ON C.ID_CATEGORA=A.CATEGORA 
  JOIN PROVEEDOR P ON P.ID_PROVEEDOR=A.PROVEEDOR 
  WHERE D.HOMENAJEADO=p_id_miembro AND R.ESTADO_REGALO='REBUT'
  ORDER BY D.HOMENAJEADO,D.ARTCULO ASC;

  LOOP
    FETCH CUR INTO ARTCULO,DESCRIPCIN,PRECIO,CATEGORA,PROVEEDOR;
    EXIT
  WHEN CUR%NOTFOUND;
    resultado.extend;
    resultado(i):= TY_R5D(ARTCULO,DESCRIPCIN,PRECIO,CATEGORA,PROVEEDOR);
    i:= i+1;
  END LOOP;
  CLOSE CUR;
  
  -- terminado ok
  RSP:= 'OK';
  
  -- llamada al procedimiento que carga la tabla de log
  SP_ALTA_LOG('CONSULTAD', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  IF (RSP_LOG!='OK') THEN
    RSP      :='No ha sido posible dar de alta el log: '|| RSP_LOG;
    ROLLBACK;
  END IF;
  RETURN resultado;
  
EXCEPTION
WHEN E_CONOCIDA THEN
  ROLLBACK;
  RSP:= 'ERROR CONSULTAD: ' || RSP;
  SP_ALTA_LOG('ConsultaA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
WHEN OTHERS THEN
  ROLLBACK;
  v_err_msg := SUBSTR (SQLERRM, 1, 200);
  RSP       :='ERROR CONSULTAD: ' || v_err_msg;
  SP_ALTA_LOG('CONSULTAD', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
  
END CONSULTAD;
/ 