/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R5E AS OBJECT
  (
   NOMBRE      VARCHAR2(50),
   APELLIDO1   VARCHAR2(50),
   APELLIDO2   VARCHAR2(50),
   IMPORTE     NUMBER
    );
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R5E AS TABLE OF TY_R5E;
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION CONSULTAE
  
RETURN TA_R5E
IS
  PRAGMA AUTONOMOUS_TRANSACTION;
  /******************************************************************************
  Nombre:                ConsultaE
  Obejtivo:              Obtener el listado de personas que se encuentran por encima 
                         de la media a nivel de dinero que han destinado a regalos grupales.
  Parmetros de entrada:  sin parmetros
  Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
  'ERROR: tipo de error' si ha fallado
  ******************************************************************************/
  RSP                  VARCHAR2 (200);
  RSP_LOG              VARCHAR2 (200);
  v_parametros_entrada VARCHAR2 (200);
  v_err_msg            VARCHAR2 (200);
  i                    INTEGER:=1;
  resultado TA_R5E :=TA_R5E();
  NOMBRE               VARCHAR2(50);
  APELLIDO1            VARCHAR2(50);
  APELLIDO2            VARCHAR2(50);
  IMPORTE              NUMBER;
  V_SOCIOS             NUMBER;
  V_CANTIDAD           NUMBER;
  V_MEDIA              NUMBER;

type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA EXCEPTION;

  BEGIN
  v_parametros_entrada:=' ';
  
  OPEN CUR FOR 
  SELECT M.NOMBRE AS NOMBRE,M.APELLIDO1 AS APELLIDO1,M.APELLIDO2 AS APELLIDO2, SUM(T.CANTIDAD) AS IMPORTE
  FROM TRANSACCIN T JOIN MIEMBRO M ON T.APORTADOR=M.ID_MIEMBRO
  GROUP BY M.NOMBRE,M.APELLIDO1,M.APELLIDO2 
  ORDER BY IMPORTE DESC; 
  
  --Calculamos el nmero de socios de la red
  SELECT COUNT(*) INTO V_SOCIOS FROM MIEMBRO;
  
  --Calculamos el gasto total
  SELECT SUM(CANTIDAD)INTO V_CANTIDAD FROM TRANSACCIN;
  
  --Calculamos la media de gasto 
  V_MEDIA:=V_SOCIOS/V_CANTIDAD;
    
  LOOP
    FETCH CUR INTO NOMBRE, APELLIDO1, APELLIDO2,IMPORTE;
    EXIT
  WHEN CUR%NOTFOUND;
   IF IMPORTE>V_MEDIA THEN
    resultado.extend;
    resultado(i):= TY_R5E(NOMBRE,APELLIDO1,APELLIDO2,IMPORTE);
    i:= i+1;
   END IF;
  END LOOP;
  CLOSE CUR;
  
  -- terminado ok
  RSP:= 'OK';
  
  -- llamada al procedimiento que carga la tabla de log
  SP_ALTA_LOG('CONSULTAE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  IF (RSP_LOG!='OK') THEN
    RSP      :='No ha sido posible dar de alta el log: '|| RSP_LOG;
    ROLLBACK;
  END IF;
  RETURN resultado;
  
EXCEPTION
WHEN E_CONOCIDA THEN
  ROLLBACK;
  RSP:= 'ERROR CONSULTAE ' || RSP;
  SP_ALTA_LOG('CONSULTAE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
WHEN OTHERS THEN
  ROLLBACK;
  v_err_msg := SUBSTR (SQLERRM, 1, 200);
  RSP       :='ERROR CONSULTAE: ' || v_err_msg;
  SP_ALTA_LOG('CONSULTAE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
  
END CONSULTAE;
/ 