/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R5F AS OBJECT
  (
   TOTAL       NUMBER,
   CIUDAD      VARCHAR2(30)
    );
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R5F AS TABLE OF TY_R5F;
  /
  /*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION CONSULTAF(
  P_AO IN INTEGER)
  
RETURN TA_R5F
IS
  PRAGMA AUTONOMOUS_TRANSACTION;
  /******************************************************************************
  Nombre:                ConsultaF
  Obejtivo:              Dado un ao el listado de las 10 ciudades donde ms regalos
                         han recibido personas homenajeadas durante el ao indicado, as
                         como el importe correspondiente a la suma de todos los regalos 
                         para cada una de estas 10 ciudades.
  Parmetros de entrada:  p_ao
  Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
  'ERROR: tipo de error' si ha fallado
  ******************************************************************************/
  RSP                  VARCHAR2 (200);
  RSP_LOG              VARCHAR2 (200);
  v_parametros_entrada VARCHAR2 (200);
  v_err_msg            VARCHAR2 (200);
  i                    INTEGER:=1;
  resultado TA_R5F :=TA_R5F();
  VECES                NUMBER(4);
  V_CIUDAD             VARCHAR2(30);
  TOTAL                NUMBER;
  
type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA EXCEPTION;

  BEGIN
    v_parametros_entrada:='p_ao: '|| TO_CHAR(P_AO);
    
    -- Comprueba si el parmetro de entrada sea nulo
    IF (P_AO IS NULL) THEN
      RSP:='El campo ao no puede ser nulo';
      RAISE E_CONOCIDA;
    END IF;
   
  OPEN CUR FOR 
    SELECT CIUDAD FROM(
    SELECT COUNT(CIUDAD)AS VECES,CIUDAD FROM
    (SELECT M.CIUDAD AS CIUDAD, R.ID_REGALO FROM TRANSACCIN T 
    JOIN REGALA R ON T.REGALO=R.ID_REGALO 
    JOIN DESEA D ON R.DESEO=D.ID_DESEO
    JOIN MIEMBRO M ON D.HOMENAJEADO=M.ID_MIEMBRO
    WHERE R.ESTADO_REGALO='REBUT'AND 
    EXTRACT(YEAR FROM R.FECHA_FINAL_RECAUDACIN)= P_AO
    GROUP BY M.CIUDAD,R.ID_REGALO
    ORDER BY (M.CIUDAD) ASC) GROUP BY CIUDAD ORDER BY VECES DESC) WHERE ROWNUM<=10;  
  
  LOOP
    FETCH CUR INTO V_CIUDAD;
    
    SELECT SUM(T.CANTIDAD) INTO TOTAL FROM TRANSACCIN T 
    JOIN REGALA R ON T.REGALO=R.ID_REGALO 
    JOIN DESEA D ON R.DESEO=D.ID_DESEO
    JOIN MIEMBRO M ON D.HOMENAJEADO=M.ID_MIEMBRO
    WHERE M.CIUDAD=V_CIUDAD;
    
    EXIT WHEN CUR%NOTFOUND;
    
    resultado.extend;
    resultado(i):= TY_R5F(TOTAL,V_CIUDAD);
    
    i:= i+1;
  
  END LOOP;
  CLOSE CUR;
  
  -- terminado ok
  RSP:= 'OK';
  
  -- llamada al procedimiento que carga la tabla de log
  SP_ALTA_LOG('CONSULTAF', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  IF (RSP_LOG!='OK') THEN
    RSP      :='No ha sido posible dar de alta el log: '|| RSP_LOG;
    ROLLBACK;
  END IF;
  RETURN resultado;
  
EXCEPTION
WHEN E_CONOCIDA THEN
  ROLLBACK;
  RSP:= 'ERROR CONSULTAF: ' || RSP;
  SP_ALTA_LOG('CONSULTAF', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
WHEN OTHERS THEN
  ROLLBACK;
  v_err_msg := SUBSTR (SQLERRM, 1, 200);
  RSP       :='ERROR CONSULTAF: ' || v_err_msg;
  SP_ALTA_LOG('CONSULTAF', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
  RETURN NULL;
  
END CONSULTAF;
/ 